
#include "ch.h"
#include "hal.h"
#include <math.h>
#include "test.h"
#include "spi.h"
#include "i2c.h"
#include "usbcfg.h"
#include "chprintf.h"

#include "L3G.h"
#include "LSM303.h"

SerialUSBDriver SDU1;

#define ADDRESS_L3G20H_HIGH			0x6B
#define ADDRESS_LSM303D_HIGH		0x1D
#define ADDRESS_L3G20H_LOW			0x6A
#define ADDRESS_LSM303D_LOW			0x1E
#define I2CDEV_DEFAULT_READ_TIMEOUT     1000
#define I2CDEV_BUFFER_LENGTH			64
#define DEBUGIN						1

#define sampleFreq	400.0f		// Frequencia de muestreo (Hz)

#define ToRad(x) ((x) * 0.01745329252)  // *pi/180

systime_t start,stop, final;

uint8_t ADDRESS[2];//ADDRESS[0] = L3G20H address; ADDRESS[1]= LSM303D address
I2CDriver *dispositivo; // I2C1 o I2C3

volatile float beta = 0.05;
volatile float q1 = 1.0f, q2 = 0.0f, q3 = 0.0f, q4 = 0.0f;	// Quaternion del sensor relativo respecto el auxiliar
float q1_1=1.0f,q1_2= 0.0f,q1_3= 0.0f,q1_4= 0.0f;
float q2_1=1.0f,q2_2= 0.0f,q2_3= 0.0f,q2_4= 0.0f;
float q3_1=1.0f,q3_2= 0.0f,q3_3= 0.0f,q3_4= 0.0f;

int16_t gyroData[3];
double gyroCorregido[3];
uint16_t accelData[3];
double accelCorregido[3];
int16_t magData[3];
double magCorregido[3];

static const I2CConfig i2c1cfg = {
		OPMODE_I2C,
		400000,
		FAST_DUTY_CYCLE_2
};
static const I2CConfig i2c3cfg = {
		OPMODE_I2C,
		400000,
		FAST_DUTY_CYCLE_2
};

// Fast inverse square-root
float invSqrt(float x) {
	float halfx = 0.5f * x;
	float y = x;
	long i = *(long*)&y;
	i = 0x5f3759df - (i>>1);
	y = *(float*)&i;
	y = y * (1.5f - (halfx * y * y));
	return y;
}

static bool_t I2CdevwriteBytes(uint8_t devAddr, uint8_t regAddr, uint8_t length, uint8_t *data) {
	uint8_t mpu_txbuf[I2CDEV_BUFFER_LENGTH], mpu_rxbuf[1];
	msg_t rdymsg;
	if((length + 1)> I2CDEV_BUFFER_LENGTH) {
		if(DEBUGIN){chprintf((BaseSequentialStream *)&SDU1,"ERROR Leer Bytes: length + 1 > I2CDEV BUFFERLENGTH\r\n");}
		return FALSE;
	}
	mpu_txbuf[0] = regAddr;
	memcpy(mpu_txbuf + sizeof(uint8_t), data, sizeof(uint8_t) * length);

	i2cAcquireBus(dispositivo);
	rdymsg = i2cMasterTransmit(dispositivo, devAddr, mpu_txbuf, length + 1, mpu_rxbuf, 0);
	i2cReleaseBus(dispositivo);
	if(rdymsg == RDY_TIMEOUT || rdymsg == RDY_RESET) {
		if(DEBUGIN){chprintf((BaseSequentialStream *)&SDU1,"I2C ERROR: %d\r\n", i2cGetErrors(dispositivo));}
		return FALSE;
	}
	return TRUE;
}
static bool_t I2CdevwriteByte(uint8_t devAddr, uint8_t regAddr, uint8_t data) {
	return I2CdevwriteBytes(devAddr, regAddr, 1, &data);
}

static int8_t I2CdevreadBytes(uint8_t devAddr, uint8_t regAddr, uint8_t length, uint8_t *data, uint16_t timeout) {

	msg_t rdymsg;
	if(length > I2CDEV_BUFFER_LENGTH) {
		if(DEBUGIN){chprintf((BaseSequentialStream *)&SDU1,"ERROR readBytes: length > I2CDEV BUFFERLENGTH\r\n");}
		return FALSE;
	}
	i2cAcquireBus(dispositivo);
	rdymsg = i2cMasterTransmitTimeout(dispositivo, devAddr, &regAddr, 1, data, length, MS2ST(timeout));
	i2cReleaseBus(dispositivo);
	if(rdymsg == RDY_TIMEOUT || rdymsg == RDY_RESET) {
		if(DEBUGIN){chprintf((BaseSequentialStream *)&SDU1,"I2C ERROR: %d\r\n", i2cGetErrors(dispositivo));}
		return FALSE;
	}
	return TRUE;
}
static int8_t I2CdevreadByte(uint8_t devAddr, uint8_t regAddr, uint8_t *data, uint16_t timeout) {
	return I2CdevreadBytes(devAddr, regAddr, 1, data, timeout);
}
static void initGyro(void)
{

	//	Sensibilidad del gyroscopio		Factor de conbersion (mdps/digit)
	//		245						8.75
	//		500						17.50
	//		2000						70.00

	I2CdevwriteByte(ADDRESS[0], L3G_CTRL_REG2, 0x00);
	I2CdevwriteByte(ADDRESS[0], L3G_CTRL_REG3, 0x00);
	I2CdevwriteByte(ADDRESS[0], L3G_CTRL_REG4, 0x20);
	I2CdevwriteByte(ADDRESS[0], L3G_CTRL_REG5, 0x02);
	I2CdevwriteByte(ADDRESS[0], L3G_CTRL_REG1, 0x0F);
	chThdSleepMilliseconds(100);

}
static void iniLSM303D(void){
	//Accelerometro
	// Pagina 10 datasheet
	// Sensibilidad magnetica     Factor de conversion (mg/LSB)			 HEX
	// 		FS=2gauss 					0.061							0x00
	// 		FS=4gauss					0.122							0x08
	//		FS=6gauss					0.183							0x10
	//		FS=8gauss					0.244							0x18
	//		FS=16gauss					0.732							0x20
	//Magnetrometro
	// Sensibilidad magnetica     Factor de conversion (mgauss/LSB)			HEX
	// 		FS=2gauss 					0.080								0x00
	// 		FS=4gauss					0.160								0x20
	//		FS=8gauss					0.320								0x40
	//		FS=12gauss					0.479								0x60
	I2CdevwriteByte(ADDRESS[1], LSM303_CTRL_REG1_A, 0b01010111);
	I2CdevwriteByte(ADDRESS[1], LSM303_CTRL_REG2_A, 0x20);// Accelerometro
	I2CdevwriteByte(ADDRESS[1], LSM303_CTRL_REG5_A, 0b01100100);
	I2CdevwriteByte(ADDRESS[1], LSM303_CTRL_REG6_A, 0b00100000);
	I2CdevwriteByte(ADDRESS[1], LSM303_CTRL_REG7_A, 0x60);// Magnetrometro
	chThdSleepMilliseconds(1000);
}
static uint8_t readGyro(int16_t* data)
{
	uint8_t out[6];
	int8_t res=I2CdevreadBytes(ADDRESS[0], L3G_OUT_X_L| (1 << 7), 6, out, I2CDEV_DEFAULT_READ_TIMEOUT);
	if(res==TRUE){
		data[0] = (int16_t)(out[1] << 8) | out[0];
		data[1] = (int16_t)(out[3] << 8) | out[2];
		data[2] = (int16_t)(out[5] << 8) | out[4];

		return 1;
	}else{
		return 0;
	}
}
static uint8_t readAccel(int16_t* data)
{
	uint8_t out[6];
	int8_t res=I2CdevreadBytes(ADDRESS[1], LSM303_OUT_X_L_A | (1 << 7), 6, out, I2CDEV_DEFAULT_READ_TIMEOUT);
	if(res==TRUE){
		data[0] = (int16_t)(out[1] << 8) | out[0];
		data[1] = (int16_t)(out[3] << 8) | out[2];
		data[2] = (int16_t)(out[5] << 8) | out[4];

		return 1;
	}else{
		return 0;
	}
}
static uint8_t readMag(int16_t* data)
{
	uint8_t out[6];
		int8_t res=I2CdevreadBytes(ADDRESS[1], LSM303_OUT_X_H_M | (1 << 7), 6, out, I2CDEV_DEFAULT_READ_TIMEOUT);
		if(res==TRUE){
			data[0] = (int16_t)(out[1] << 8) | out[0];
			data[1] = (int16_t)(out[3] << 8) | out[2];
			data[2] = (int16_t)(out[5] << 8) | out[4];

			return 1;
		}else{
			return 0;
		}
}
void AHRS(float gx, float gy, float gz, float ax, float ay, float az, float mx, float my, float mz) {


	float recipNorm;
	float s1, s2, s3, s4;
	float qDiff1, qDiff2, qDiff3, qDiff4;
	float hx, hy;
	float _2q1mx, _2q1my, _2q1mz, _2q2mx, _2bx, _2bz, _4bx, _4bz, _2q1, _2q2, _2q3, _2q4, _2q1q3, _2q3q4, q1q1, q1q2, q1q3, q1q4, q2q2, q2q3, q2q4, q3q3, q3q4, q4q4;

	// Utilizaremos el algoritmo de la IMU por si las lecturas del magnetrometro son incorrectas
	if((mx == 0.0f) && (my == 0.0f) && (mz == 0.0f)) {
		IMU(gx, gy, gz, ax, ay, az);
		return;
	}

	// Tasa de cambio del quaternion debido al giroscopio, ecuaciones (11) y (12)
	qDiff1 = 0.5f * (-q2 * gx - q3 * gy - q4 * gz);
	qDiff2 = 0.5f * (q1 * gx + q3 * gz - q4 * gy);
	qDiff3 = 0.5f * (q1 * gy - q2 * gz + q4 * gx);
	qDiff4 = 0.5f * (q1 * gz + q2 * gy - q3 * gx);

	// Calcular la contribucin del accelerometro solo si este no es invalido,
	// evitamos los NaN en la normalizacin de la medida del accelerometro
	if(!((ax == 0.0f) && (ay == 0.0f) && (az == 0.0f))) {

		// Normalizacin de la medida del accelerometro
		recipNorm = invSqrt(ax * ax + ay * ay + az * az);
		ax *= recipNorm;
		ay *= recipNorm;
		az *= recipNorm;

		// Normalizacin de la medida del magnetrometro
		recipNorm = invSqrt(mx * mx + my * my + mz * mz);
		mx *= recipNorm;
		my *= recipNorm;
		mz *= recipNorm;
		// Creacin de variables auxiliares para evitar
		// realizar la misma operacin multiples veces.
		_2q1mx = 2.0f * q1 * mx;
		_2q1my = 2.0f * q1 * my;
		_2q1mz = 2.0f * q1 * mz;
		_2q2mx = 2.0f * q2 * mx;
		_2q1 = 2.0f * q1;
		_2q2 = 2.0f * q2;
		_2q3 = 2.0f * q3;
		_2q4 = 2.0f * q4;
		_2q1q3 = 2.0f * q1 * q3;
		_2q3q4 = 2.0f * q3 * q4;
		q1q1 = q1 * q1;
		q1q2 = q1 * q2;
		q1q3 = q1 * q3;
		q1q4 = q1 * q4;
		q2q2 = q2 * q2;
		q2q3 = q2 * q3;
		q2q4 = q2 * q4;
		q3q3 = q3 * q3;
		q3q4 = q3 * q4;
		q4q4 = q4 * q4;

		// Calculo de la compensacin de la distorsion del campo magntico de la tierra
		// Ecuaciones (45) y (46)
		hx = mx * q1q1 - _2q1my * q4 + _2q1mz * q3 + mx * q2q2 + _2q2 * my * q3 + _2q2 * mz * q4 - mx * q3q3 - mx * q4q4;
		hy = _2q1mx * q4 + my * q1q1 - _2q1mz * q2 + _2q2mx * q3 - my * q2q2 + my * q3q3 + _2q3 * mz * q4 - my * q4q4;
		_2bz = -_2q1mx * q3 + _2q1my * q2 + mz * q1q1 + _2q2mx * q4 - mz * q2q2 + _2q3 * my * q4 - mz * q3q3 + mz * q4q4; // = hz
		_2bx = sqrt(hx * hx + hy * hy);

		_4bx = 2.0f * _2bx;
		_4bz = 2.0f * _2bz;

		// Correccin del algoritmo de gradiente descendiente Jtran*f ecuacin (34)
		s1 = -_2q3 * (2.0f * q2q4 - _2q1q3 - ax) + _2q2 * (2.0f * q1q2 + _2q3q4 - ay) - _2bz * q3 * (_2bx * (0.5f - q3q3 - q4q4) + _2bz * (q2q4 - q1q3) - mx) + (-_2bx * q4 + _2bz * q2) * (_2bx * (q2q3 - q1q4) + _2bz * (q1q2 + q3q4) - my) + _2bx * q3 * (_2bx * (q1q3 + q2q4) + _2bz * (0.5f - q2q2 - q3q3) - mz);
		s2 = _2q4 * (2.0f * q2q4 - _2q1q3 - ax) + _2q1 * (2.0f * q1q2 + _2q3q4 - ay) - 4.0f * q2 * (1 - 2.0f * q2q2 - 2.0f * q3q3 - az) + _2bz * q4 * (_2bx * (0.5f - q3q3 - q4q4) + _2bz * (q2q4 - q1q3) - mx) + (_2bx * q3 + _2bz * q1) * (_2bx * (q2q3 - q1q4) + _2bz * (q1q2 + q3q4) - my) + (_2bx * q4 - _4bz * q2) * (_2bx * (q1q3 + q2q4) + _2bz * (0.5f - q2q2 - q3q3) - mz);
		s3 = -_2q1 * (2.0f * q2q4 - _2q1q3 - ax) + _2q4 * (2.0f * q1q2 + _2q3q4 - ay) - 4.0f * q3 * (1 - 2.0f * q2q2 - 2.0f * q3q3 - az) + (-_4bx * q3 - _2bz * q1) * (_2bx * (0.5f - q3q3 - q4q4) + _2bz * (q2q4 - q1q3) - mx) + (_2bx * q2 + _2bz * q4) * (_2bx * (q2q3 - q1q4) + _2bz * (q1q2 + q3q4) - my) + (_2bx * q1 - _4bz * q3) * (_2bx * (q1q3 + q2q4) + _2bz * (0.5f - q2q2 - q3q3) - mz);
		s4 = _2q2 * (2.0f * q2q4 - _2q1q3 - ax) + _2q3 * (2.0f * q1q2 + _2q3q4 - ay) + (-_4bx * q4 + _2bz * q2) * (_2bx * (0.5f - q3q3 - q4q4) + _2bz * (q2q4 - q1q3) - mx) + (-_2bx * q1 + _2bz * q3) * (_2bx * (q2q3 - q1q4) + _2bz * (q1q2 + q3q4) - my) + _2bx * q2 * (_2bx * (q1q3 + q2q4) + _2bz * (0.5f - q2q2 - q3q3) - mz);
		recipNorm = invSqrt(s1 * s1 + s2 * s2 + s3 * s3 + s4 * s4); // Normalizacin de la contribucin del giroscopio y del magnetrometro ecuacin (44)
		s1 *= recipNorm;
		s2 *= recipNorm;
		s3 *= recipNorm;
		s4 *= recipNorm;

		// Aplicacin de la correccin al cuaternin del giroscopio ecuacin (43)
		qDiff1 -= beta * s1;
		qDiff2 -= beta * s2;
		qDiff3 -= beta * s3;
		qDiff4 -= beta * s4;
	}

	// Integracin del cuaternion ecuacin (42)
	q1 += qDiff1 * (1.0f / sampleFreq);
	q2 += qDiff2 * (1.0f / sampleFreq);
	q3 += qDiff3 * (1.0f / sampleFreq);
	q4 += qDiff4 * (1.0f / sampleFreq);

	// Normalizacin del cuaternin
	recipNorm = invSqrt(q1 * q1 + q2 * q2 + q3 * q3 + q4 * q4);
	q1 *= recipNorm;
	q2 *= recipNorm;
	q3 *= recipNorm;
	q4 *= recipNorm;
}
void IMU(float gx, float gy, float gz, float ax, float ay, float az) {
	float recipNorm;
	float s1, s2, s3, s4;
	float qDiff1, qDiff2, qDiff3, qDiff4;
	float _2q1, _2q2, _2q3, _2q4, _4q1, _4q2, _4q3 ,_8q2, _8q3, q1q1, q2q2, q3q3, q4q4;

	qDiff1 = 0.5f * (-q2 * gx - q3 * gy - q4 * gz);
	qDiff2 = 0.5f * (q1 * gx + q3 * gz - q4 * gy);
	qDiff3 = 0.5f * (q1 * gy - q2 * gz + q4 * gx);
	qDiff4 = 0.5f * (q1 * gz + q2 * gy - q3 * gx);

	if(!((ax == 0.0f) && (ay == 0.0f) && (az == 0.0f))) {

		recipNorm = invSqrt(ax * ax + ay * ay + az * az);
		ax *= recipNorm;
		ay *= recipNorm;
		az *= recipNorm;

		_2q1 = 2.0f * q1;
		_2q2 = 2.0f * q2;
		_2q3 = 2.0f * q3;
		_2q4 = 2.0f * q4;
		_4q1 = 4.0f * q1;
		_4q2 = 4.0f * q2;
		_4q3 = 4.0f * q3;
		_8q2 = 8.0f * q2;
		_8q3 = 8.0f * q3;
		q1q1 = q1 * q1;
		q2q2 = q2 * q2;
		q3q3 = q3 * q3;
		q4q4 = q4 * q4;

		s1 = _4q1 * q3q3 + _2q3 * ax + _4q1 * q2q2 - _2q2 * ay;
		s2 = _4q2 * q4q4 - _2q4 * ax + 4.0f * q1q1 * q2 - _2q1 * ay - _4q2 + _8q2 * q2q2 + _8q2 * q3q3 + _4q2 * az;
		s3 = 4.0f * q1q1 * q3 + _2q1 * ax + _4q3 * q4q4 - _2q4 * ay - _4q3 + _8q3 * q2q2 + _8q3 * q3q3 + _4q3 * az;
		s4 = 4.0f * q2q2 * q4 - _2q2 * ax + 4.0f * q3q3 * q4 - _2q3 * ay;
		recipNorm = invSqrt(s1 * s1 + s2 * s2 + s3 * s3 + s4 * s4);
		s1 *= recipNorm;
		s2 *= recipNorm;
		s3 *= recipNorm;
		s4 *= recipNorm;

		qDiff1 -= beta * s1;
		qDiff2 -= beta * s2;
		qDiff3 -= beta * s3;
		qDiff4 -= beta * s4;
	}

	q1 += qDiff1 * (1.0f / sampleFreq);
	q2 += qDiff2 * (1.0f / sampleFreq);
	q3 += qDiff3 * (1.0f / sampleFreq);
	q4 += qDiff4 * (1.0f / sampleFreq);

	recipNorm = invSqrt(q1 * q1 + q2 * q2 + q3 * q3 + q4 * q4);
	q1 *= recipNorm;
	q2 *= recipNorm;
	q3 *= recipNorm;
	q4 *= recipNorm;
}
void correctorDeMuestras(double* accelOffset, double* gyroOffset, double* magOffset){
	accelCorregido[0] = ((double)accelData[0] - accelOffset[0]);
	accelCorregido[1] = ((double)accelData[1] - accelOffset[1]);
	accelCorregido[2] = ((double)accelData[2] - accelOffset[2]);
	gyroCorregido[0] = ToRad(((double)gyroData[0] - gyroOffset[0]));
	gyroCorregido[1] = ToRad(((double)gyroData[1] - gyroOffset[1]));
	gyroCorregido[2] = ToRad(((double)gyroData[2] - gyroOffset[2]));
	magCorregido[0] = ((double)magData[0] - magOffset[0]);
	magCorregido[1] = ((double)magData[1] - magOffset[1]);
	magCorregido[2] = ((double)magData[2] - magOffset[2]);
}

int main(void) {

	/*
	* Inicialitzaci del sistema.
	*
	*/
	halInit();
	chSysInit();

	/*
	* Inicialitzaci del diver del puerto serie.
	*/
	sduObjectInit(&SDU1);
	sduStart(&SDU1, &serusbcfg);

	usbDisconnectBus(serusbcfg.usbp);
	chThdSleepMilliseconds(1000);
	usbStart(serusbcfg.usbp, &usbcfg);
	usbConnectBus(serusbcfg.usbp);
	/*
	* Inicialitzaci del diver I2C1.
	*/
	i2cStart(&I2CD1, &i2c1cfg);
	palSetPadMode(GPIOB, 8, PAL_MODE_ALTERNATE(4) | PAL_STM32_OTYPE_OPENDRAIN);   // SCL
	palSetPadMode(GPIOB, 9, PAL_MODE_ALTERNATE(4) | PAL_STM32_OTYPE_OPENDRAIN);   // SDA
	chThdSleepMilliseconds(1000);
	/*
	* Inicialitzaci del diver I2C2.
	*/
	i2cStart(&I2CD2, &i2c3cfg);
	palSetPadMode(GPIOB, 10, PAL_MODE_ALTERNATE(4) | PAL_STM32_OTYPE_OPENDRAIN);   // SCL
	palSetPadMode(GPIOB, 11, PAL_MODE_ALTERNATE(4) | PAL_STM32_OTYPE_OPENDRAIN);   // SDA
	chThdSleepMilliseconds(1000);
	/*
	* Inicialitzaci del primer sensor con SA0 = LOW que esta en el I2C2.
	*/
	ADDRESS[0]=ADDRESS_L3G20H_LOW;
	ADDRESS[1]=ADDRESS_LSM303D_LOW;
	dispositivo=&I2CD2;
	initGyro();
	iniLSM303D();
	chThdSleepMilliseconds(10);
	/*
	* Inicialitzaci del sensor que esta en el I2C1.
	*/
	ADDRESS[0]=ADDRESS_L3G20H_LOW;
	ADDRESS[1]=ADDRESS_LSM303D_LOW;
	dispositivo=&I2CD1;
	initGyro();
	iniLSM303D();
	chThdSleepMilliseconds(10);
	/*
	* Inicialitzaci del primer sensor con SA0 = HIGH que esta en el I2C2.
	*/
	ADDRESS[0]=ADDRESS_L3G20H_HIGH;
	ADDRESS[1]=ADDRESS_LSM303D_HIGH;
	dispositivo=&I2CD2;
	initGyro();
	iniLSM303D();
	chThdSleepMilliseconds(10);


	double accelOffset_1_1[]={0,0,0};	//Offset del accelerometro n1 I2C1
	double accelOffset_3_1[]={0,0,0};	//Offset del accelerometro n1 I2C3
	double accelOffset_3_2[]={0,0,0};	//Offset del accelerometro n2 I2C3
	double gyroOffset_1_1[]={0,0,0};	//Offset del giroscopio n1 I2C1
	double gyroOffset_3_1[]={0,0,0};	//Offset del giroscopio n1 I2C3
	double gyroOffset_3_2[]={0,0,0};	//Offset del giroscopio n2 I2C3
	double magOffset_1_1[]={0,0,0};	//Offset del magnetometro n1 I2C1
	double magOffset_3_1[]={0,0,0};	//Offset del magnetometro n1 I2C3
	double magOffset_3_2[]={0,0,0};	//Offset del magnetometro n2 I2C3

	/*
	* Calculo del Offset del sensor I2C1
	*/
	int32_t i;	//Variable auxiliar
	int32_t muestras=30;	//Numero de muestras para determinar el offset

	/*
	* Calculo del Offset del sensor n1 I2C2
	*/
	ADDRESS[0]=ADDRESS_L3G20H_LOW;
	ADDRESS[1]=ADDRESS_LSM303D_LOW;
	dispositivo=&I2CD2;
	for (i = 0; i < muestras;i++){
		readGyro(&gyroData);
		readAccel(&accelData);
		readMag(&magData);
		gyroOffset_3_1[0] += (double)gyroData[0]/muestras;
		gyroOffset_3_1[1] += (double)gyroData[1]/muestras;
		gyroOffset_3_1[2] += (double)gyroData[2]/muestras;
		accelOffset_3_1[0] += (double)accelData[0]/muestras;
		accelOffset_3_1[1] += (double)accelData[1]/muestras;
		accelOffset_3_1[2] += (double)accelData[2]/muestras;
		magOffset_3_1[0] += (double)magData[0]/muestras;
		magOffset_3_1[1] += (double)magData[1]/muestras;
		magOffset_3_1[2] += (double)magData[2]/muestras;
		chThdSleepMilliseconds(10);
	}


	ADDRESS[0]=ADDRESS_L3G20H_LOW;
	ADDRESS[1]=ADDRESS_LSM303D_LOW;
	dispositivo=&I2CD1;

	for (i = 0; i < muestras;i++){
		readGyro(&gyroData);
		readAccel(&accelData);
		readMag(&magData);
		gyroOffset_1_1[0] += (double)gyroData[0]/muestras;
		gyroOffset_1_1[1] += (double)gyroData[1]/muestras;
		gyroOffset_1_1[2] += (double)gyroData[2]/muestras;
		accelOffset_1_1[0] += (double)accelData[0]/muestras;
		accelOffset_1_1[1] += (double)accelData[1]/muestras;
		accelOffset_1_1[2] += (double)accelData[2]/muestras;
		magOffset_1_1[0] += (double)magData[0]/muestras;
		magOffset_1_1[1] += (double)magData[1]/muestras;
		magOffset_1_1[2] += (double)magData[2]/muestras;
		chThdSleepMilliseconds(10);
	}

	/*
	* Calculo del Offset del sensor n2 I2C3
	*/
	ADDRESS[0]=ADDRESS_L3G20H_HIGH;
	ADDRESS[1]=ADDRESS_LSM303D_HIGH;
	dispositivo=&I2CD2;
	for (i = 0; i < muestras;i++){
		readGyro(&gyroData);
		readAccel(&accelData);
		readMag(&magData);
		gyroOffset_3_2[0] += (double)gyroData[0]/muestras;
		gyroOffset_3_2[1] += (double)gyroData[1]/muestras;
		gyroOffset_3_2[2] += (double)gyroData[2]/muestras;
		accelOffset_3_2[0] += (double)accelData[0]/muestras;
		accelOffset_3_2[1] += (double)accelData[1]/muestras;
		accelOffset_3_2[2] += (double)accelData[2]/muestras;
		magOffset_3_2[0] += (double)magData[0]/muestras;
		magOffset_3_2[1] += (double)magData[1]/muestras;
		magOffset_3_2[2] += (double)magData[2]/muestras;
		chThdSleepMilliseconds(10);
	}
	int32_t count=0;
	int32_t mean=0;
	while (TRUE) {
		start =chTimeNow();
		ADDRESS[0]=ADDRESS_L3G20H_LOW;
		ADDRESS[1]=ADDRESS_LSM303D_LOW;
		dispositivo=&I2CD2;
		if (readGyro(&gyroData)& readAccel(&accelData) & readMag(&magData)) {
			correctorDeMuestras(accelOffset_3_1,gyroOffset_3_1,magOffset_3_1);
			q1=q2_1;q2=q2_2;q3=q2_3;q4=q2_4;
			AHRS(gyroCorregido[0], gyroCorregido[1], gyroCorregido[2], accelCorregido[0],accelCorregido[1], accelCorregido[2], magCorregido[0], magCorregido[1], magCorregido[2]);
			q2_1=q1;q2_2=q2;q2_3=q3;q2_4=q4;
			//chprintf((BaseSequentialStream *)&SDU1, "1,%f,%f,%f,%f", q2_1,q2_2, q2_3,q2_4);

		}

		ADDRESS[0]=ADDRESS_L3G20H_LOW;
		ADDRESS[1]=ADDRESS_LSM303D_LOW;
		dispositivo=&I2CD1;
		if (readGyro(&gyroData)& readAccel(&accelData) & readMag(&magData)) {
			correctorDeMuestras(accelOffset_1_1,gyroOffset_1_1,magOffset_1_1);
			q1=q1_1;q2=q1_2;q3=q1_3;q4=q1_4;
			AHRS(gyroCorregido[0], gyroCorregido[1], gyroCorregido[2], accelCorregido[0],accelCorregido[1], accelCorregido[2], magCorregido[0], magCorregido[1], magCorregido[2]);
			q1_1=q1;q1_2=q2;q1_3=q3;q1_4=q4;
			//chprintf((BaseSequentialStream *)&SDU1, ",2,%f,%f,%f,%f", q1_1, q1_2, q1_3,q1_4);

		}



		ADDRESS[0]=ADDRESS_L3G20H_HIGH;
		ADDRESS[1]=ADDRESS_LSM303D_HIGH;
		dispositivo=&I2CD2;
		if (readGyro(&gyroData)& readAccel(&accelData) & readMag(&magData)) {
			correctorDeMuestras(accelOffset_3_2,gyroOffset_3_2,magOffset_3_2);
			q1=q3_1;q2=q3_2;q3=q3_3;q4=q3_4;
			AHRS(gyroCorregido[0], gyroCorregido[1], gyroCorregido[2], accelCorregido[0],accelCorregido[1], accelCorregido[2], magCorregido[0], magCorregido[1], magCorregido[2]);
			q3_1=q1;q3_2=q2;q3_3=q3;q3_4=q4;
			//chprintf((BaseSequentialStream *)&SDU1, ",3,%f,%f,%f,%f\r\n", q3_1, q3_2, q3_3,q3_4);

		}

		mean += chTimeElapsedSince(start);
		count++;
		chprintf((BaseSequentialStream *)&SDU1," Time : %ld\r\n"	,US2ST(mean)/count);
    }
 return -1;
}
